@@ -43,7 +43,6 @@
 
 #include "../../common/header/common.h"
 #include "../../common/header/glob.h"
-#include "../generic/header/input.h"
 
 // Pointer to game library
 static void *game_library;
@@ -351,9 +350,6 @@ Sys_GetGameAPI(void *parms)
 	const char *gamename = "game.so";
 #endif
 
-	setreuid(getuid(), getuid());
-	setegid(getgid());
-
 	if (game_library)
 	{
 		Com_Error(ERR_FATAL, "Sys_GetGameAPI without Sys_UnloadingGame");
@@ -485,6 +481,38 @@ Sys_GetHomeDir(void)
 	return gdir;
 }
 
+void
+Sys_Remove(const char *path)
+{
+	remove(path);
+}
+
+int
+Sys_Rename(const char *from, const char *to)
+{
+	return rename(from, to);
+}
+
+void
+Sys_RemoveDir(const char *path)
+{
+	char filepath[MAX_OSPATH];
+	DIR *directory = opendir(path);
+	struct dirent *file;
+
+	if (Sys_IsDir(path))
+	{
+		while ((file = readdir(directory)) != NULL)
+		{
+			snprintf(filepath, MAX_OSPATH, "%s/%s", path, file->d_name);
+			Sys_Remove(filepath);
+		}
+
+		closedir(directory);
+		Sys_Remove(path);
+	}
+}
+
 /* ================================================================ */
 
 void *
