@@ -30,12 +30,23 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define NUMSTACKEDGES		2048
 #define NUMSTACKSURFACES	1024
 #define MAXALIASVERTS		2048
+#define MAXLIGHTS		1024 // allow some very large lightmaps
 
 viddef_t	vid;
 pixel_t		*vid_buffer = NULL;
+static pixel_t	*swap_buffers = NULL;
+static pixel_t	*swap_frames[2] = {NULL, NULL};
+static int	swap_current = 0;
 espan_t		*vid_polygon_spans = NULL;
 pixel_t		*vid_colormap = NULL;
 pixel_t		*vid_alphamap = NULL;
+static int	vid_minu, vid_minv, vid_maxu, vid_maxv;
+static int	vid_zminu, vid_zminv, vid_zmaxu, vid_zmaxv;
+
+// last position  on map
+static vec3_t	lastvieworg;
+static vec3_t	lastviewangles;
+qboolean	fastmoving;
 
 refimport_t	ri;
 
@@ -68,11 +79,15 @@ float	r_time1;
 int	r_numallocatededges;
 int	r_numallocatedverts;
 int	r_numallocatedtriangles;
+int	r_numallocatedlights;
+int	r_numallocatededgebasespans;
 float	r_aliasuvscale = 1.0;
 qboolean	r_outofsurfaces;
 qboolean	r_outofedges;
 qboolean	r_outofverts;
 qboolean	r_outoftriangles;
+qboolean	r_outoflights;
+qboolean	r_outedgebasespans;
 
 qboolean	r_dowarp;
 
@@ -176,9 +191,9 @@ static cvar_t	*r_lockpvs;
 // FIXME: make into one big structure, like cl or sv
 // FIXME: do separately for refresh engine and driver
 
-float	d_sdivzstepu, d_tdivzstepu, d_zistepu;
-float	d_sdivzstepv, d_tdivzstepv, d_zistepv;
-float	d_sdivzorigin, d_tdivzorigin, d_ziorigin;
+float	d_sdivzstepu, d_tdivzstepu;
+float	d_sdivzstepv, d_tdivzstepv;
+float	d_sdivzorigin, d_tdivzorigin;
 
 int	sadjust, tadjust, bbextents, bbextentt;
 
@@ -187,14 +202,138 @@ int		cachewidth;
 pixel_t		*d_viewbuffer;
 zvalue_t	*d_pzbuffer;
 
-qboolean	insubmodel;
-
 static void Draw_GetPalette (void);
 static void RE_BeginFrame( float camera_separation );
 static void Draw_BuildGammaTable(void);
+static void RE_FlushFrame(int vmin, int vmax);
+static void RE_CleanFrame(void);
 static void RE_EndFrame(void);
 static void R_DrawBeam(const entity_t *e);
 
+/*
+================
+VID_DamageZBuffer
+
+Mark VID Z buffer as damaged and need to be recalculated
+================
+*/
+void
+VID_DamageZBuffer(int u, int v)
+{
+	// update U
+	if (vid_zminu > u)
+	{
+		vid_zminu = u;
+	}
+	if (vid_zmaxu < u)
+	{
+		vid_zmaxu = u;
+	}
+
+	// update V
+	if (vid_zminv > v)
+	{
+		vid_zminv = v;
+	}
+	if (vid_zmaxv < v)
+	{
+		vid_zmaxv = v;
+	}
+}
+
+// clean z damage state
+static void
+VID_NoDamageZBuffer(void)
+{
+	vid_zminu = vid.width;
+	vid_zmaxu = 0;
+	vid_zminv = vid.height;
+	vid_zmaxv = 0;
+}
+
+qboolean
+VID_CheckDamageZBuffer(int u, int v, int ucount, int vcount)
+{
+	if (vid_zminv > (v + vcount) || vid_zmaxv < v)
+	{
+		// can be used previous zbuffer
+		return false;
+	}
+
+	if (vid_zminu > u && vid_zminu > (u + ucount))
+	{
+		// can be used previous zbuffer
+		return false;
+	}
+
+	if (vid_zmaxu < u && vid_zmaxu < (u + ucount))
+	{
+		// can be used previous zbuffer
+		return false;
+	}
+	return true;
+}
+
+// Need to recalculate whole z buffer
+static void
+VID_WholeDamageZBuffer(void)
+{
+	vid_zminu = 0;
+	vid_zmaxu = vid.width;
+	vid_zminv = 0;
+	vid_zmaxv = vid.height;
+}
+
+/*
+================
+VID_DamageBuffer
+
+Mark VID buffer as damaged and need to be rewritten
+================
+*/
+void
+VID_DamageBuffer(int u, int v)
+{
+	// update U
+	if (vid_minu > u)
+	{
+		vid_minu = u;
+	}
+	if (vid_maxu < u)
+	{
+		vid_maxu = u;
+	}
+	// update V
+	if (vid_minv > v)
+	{
+		vid_minv = v;
+	}
+	if (vid_maxv < v)
+	{
+		vid_maxv = v;
+	}
+}
+
+// clean damage state
+static void
+VID_NoDamageBuffer(void)
+{
+	vid_minu = vid.width;
+	vid_maxu = 0;
+	vid_minv = vid.height;
+	vid_maxv = 0;
+}
+
+// Need to rewrite whole buffer
+static void
+VID_WholeDamageBuffer(void)
+{
+	vid_minu = 0;
+	vid_maxu = vid.width;
+	vid_minv = 0;
+	vid_maxv = vid.height;
+}
+
 /*
 ================
 R_InitTurb
@@ -382,6 +521,14 @@ R_ReallocateMapBuffers (void)
 		if (r_cnumsurfs < NUMSTACKSURFACES)
 			r_cnumsurfs = NUMSTACKSURFACES;
 
+		// edge_t->surf limited size to short
+		if (r_cnumsurfs > SURFINDEX_MAX)
+		{
+			r_cnumsurfs = SURFINDEX_MAX;
+			R_Printf(PRINT_ALL, "%s: Code has limitation to surfaces count.\n",
+				 __func__);
+		}
+
 		lsurfs = malloc (r_cnumsurfs * sizeof(surf_t));
 		if (!lsurfs)
 		{
@@ -393,14 +540,46 @@ R_ReallocateMapBuffers (void)
 		surfaces = lsurfs;
 		// set limits
 		surf_max = &surfaces[r_cnumsurfs];
-		surface_p = lsurfs;
 		// surface 0 doesn't really exist; it's just a dummy because index 0
 		// is used to indicate no edge attached to surface
 		surfaces--;
 
+		surface_p = &surfaces[2];	// background is surface 1,
+						//  surface 0 is a dummy
+
 		R_Printf(PRINT_ALL, "Allocated %d surfaces\n", r_cnumsurfs);
 	}
 
+	if (!r_numallocatedlights || r_outoflights)
+	{
+		if (!blocklights)
+		{
+			free(blocklights);
+		}
+
+		if (r_outoflights)
+		{
+			r_numallocatedlights *= 2;
+			r_outoflights = false;
+		}
+
+		if (r_numallocatedlights < MAXLIGHTS)
+			r_numallocatedlights = MAXLIGHTS;
+
+		blocklights = malloc (r_numallocatedlights * sizeof(light_t));
+		if (!blocklights)
+		{
+			R_Printf(PRINT_ALL, "%s: Couldn't malloc %d bytes\n",
+				 __func__, (int)(r_numallocatedlights * sizeof(light_t)));
+			return;
+		}
+
+		// set limits
+		blocklight_max = &blocklights[r_numallocatedlights];
+
+		R_Printf(PRINT_ALL, "Allocated %d lights\n", r_numallocatedlights);
+	}
+
 	if (!r_numallocatededges || r_outofedges)
 	{
 		if (!r_edges)
@@ -487,6 +666,35 @@ R_ReallocateMapBuffers (void)
 
 		R_Printf(PRINT_ALL, "Allocated %d triangles\n", r_numallocatedtriangles);
 	}
+
+	if (!r_numallocatededgebasespans || r_outedgebasespans)
+	{
+		if (edge_basespans)
+		{
+			free(edge_basespans);
+		}
+
+		if (r_outedgebasespans)
+		{
+			r_numallocatededgebasespans *= 2;
+			r_outedgebasespans = false;
+		}
+
+		// used up to 8 * width spans for render, allocate once  before use
+		if (r_numallocatededgebasespans < vid.width * 8)
+			r_numallocatededgebasespans = vid.width * 8;
+
+		edge_basespans  = malloc(r_numallocatededgebasespans * sizeof(espan_t));
+		if (!edge_basespans)
+		{
+			R_Printf(PRINT_ALL, "%s: Couldn't malloc %d bytes\n",
+				 __func__, (int)(r_numallocatededgebasespans * sizeof(espan_t)));
+			return;
+		}
+		max_span_p = &edge_basespans[r_numallocatededgebasespans];
+
+		R_Printf(PRINT_ALL, "Allocated %d edgespans\n", r_numallocatededgebasespans);
+	}
 }
 
 
@@ -839,7 +1047,6 @@ R_DrawBEntitiesOnList (void)
 		return;
 
 	VectorCopy (modelorg, oldorigin);
-	insubmodel = true;
 
 	for (i=0 ; i<r_newrefdef.num_entities ; i++)
 	{
@@ -901,8 +1108,6 @@ R_DrawBEntitiesOnList (void)
 		VectorCopy (oldorigin, modelorg);
 		R_TransformFrustum ();
 	}
-
-	insubmodel = false;
 }
 
 /*
@@ -954,7 +1159,7 @@ R_EdgeDrawing (void)
 
 //=======================================================================
 
-static void	R_GammaCorrectAndSetPalette(const unsigned char *pal);
+static void	R_GammaCorrectAndSetPalette(const unsigned char *palette);
 
 /*
 =============
@@ -1029,6 +1234,16 @@ R_SetLightLevel (const entity_t *currententity)
 	r_lightlevel->value = 150.0 * light[0];
 }
 
+static int
+VectorCompareRound(vec3_t v1, vec3_t v2)
+{
+	if ((int)(v1[0] - v2[0]) || (int)(v1[1] - v2[1]) || (int)(v1[2] - v2[2]))
+	{
+		return 0;
+	}
+
+	return 1;
+}
 
 /*
 ================
@@ -1046,9 +1261,27 @@ RE_RenderFrame (refdef_t *fd)
 		ri.Sys_Error(ERR_FATAL, "%s: NULL worldmodel", __func__);
 	}
 
+	// Need to rerender whole frame
+	VID_WholeDamageBuffer();
+
 	VectorCopy (fd->vieworg, r_refdef.vieworg);
 	VectorCopy (fd->viewangles, r_refdef.viewangles);
 
+	// compare current position with old
+	if (!VectorCompareRound(fd->vieworg, lastvieworg) ||
+	    !VectorCompare(fd->viewangles, lastviewangles))
+	{
+		fastmoving = true;
+	}
+	else
+	{
+		fastmoving = false;
+	}
+
+	// save position for next check
+	VectorCopy (fd->vieworg, lastvieworg);
+	VectorCopy (fd->viewangles, lastviewangles);
+
 	if (r_speeds->value || r_dspeeds->value)
 		r_time1 = SDL_GetTicks();
 
@@ -1071,6 +1304,16 @@ RE_RenderFrame (refdef_t *fd)
 		de_time1 = se_time2;
 	}
 
+	if (fastmoving)
+	{
+		// redraw all
+		VID_WholeDamageZBuffer();
+	}
+	else
+	{
+		// No Z rewrite required
+		VID_NoDamageZBuffer();
+	}
 	// Draw enemies, barrel etc...
 	// Use Z-Buffer mostly in read mode only.
 	R_DrawEntitiesOnList ();
@@ -1097,10 +1340,10 @@ RE_RenderFrame (refdef_t *fd)
 		D_WarpScreen ();
 
 	if (r_dspeeds->value)
+	{
 		da_time1 = SDL_GetTicks();
-
-	if (r_dspeeds->value)
 		da_time2 = SDL_GetTicks();
+	}
 
 	// Modify the palette (when taking hit or pickup item) so all colors are modified
 	R_CalcPalette ();
@@ -1163,6 +1406,10 @@ RE_BeginFrame( float camera_separation )
 	{
 		Draw_BuildGammaTable();
 		R_GammaCorrectAndSetPalette((const unsigned char * )d_8to24table);
+		// we need redraw everything
+		VID_WholeDamageBuffer();
+		// and backbuffer should be zeroed
+		memset(swap_buffers + ((swap_current + 1)&1), 0, vid.height * vid.width * sizeof(pixel_t));
 
 		vid_gamma->modified = false;
 		sw_overbrightbits->modified = false;
@@ -1242,10 +1489,7 @@ RE_SetPalette(const unsigned char *palette)
 	byte palette32[1024];
 
 	// clear screen to black to avoid any palette flash
-	memset(vid_buffer, 0, vid.height * vid.width * sizeof(pixel_t));
-
-	// flush it to the screen
-	RE_EndFrame ();
+	RE_CleanFrame();
 
 	if (palette)
 	{
@@ -1592,11 +1836,14 @@ RE_InitContext(void *win)
 static void
 RE_ShutdownContext(void)
 {
-	if (vid_buffer)
+	if (swap_buffers)
 	{
-		free(vid_buffer);
+		free(swap_buffers);
 	}
+	swap_buffers = NULL;
 	vid_buffer = NULL;
+	swap_frames[0] = NULL;
+	swap_frames[1] = NULL;
 
 	if (sintable)
 	{
@@ -1664,6 +1911,12 @@ RE_ShutdownContext(void)
 	}
 	finalverts = NULL;
 
+	if(blocklights)
+	{
+		free(blocklights);
+	}
+	blocklights = NULL;
+
 	if(r_edges)
 	{
 		free(r_edges);
@@ -1702,7 +1955,7 @@ point math used in R_ScanEdges() overflows at width 2048 !!
 char shift_size;
 
 static void
-RE_CopyFrame (Uint32 * pixels, int pitch)
+RE_CopyFrame (Uint32 * pixels, int pitch, int vmin, int vmax)
 {
 	Uint32 *sdl_palette = (Uint32 *)sw_state.currentpalette;
 
@@ -1713,10 +1966,10 @@ RE_CopyFrame (Uint32 * pixels, int pitch)
 		Uint32	*pixels_pos;
 		pixel_t	*buffer_pos;
 
-		max_pixels = pixels + vid.height * vid.width;
-		buffer_pos = vid_buffer;
+		max_pixels = pixels + vmax;
+		buffer_pos = vid_buffer + vmin;
 
-		for (pixels_pos = pixels; pixels_pos < max_pixels; pixels_pos++)
+		for (pixels_pos = pixels + vmin; pixels_pos < max_pixels; pixels_pos++)
 		{
 			*pixels_pos = sdl_palette[*buffer_pos];
 			buffer_pos++;
@@ -1724,10 +1977,14 @@ RE_CopyFrame (Uint32 * pixels, int pitch)
 	}
 	else
 	{
-		int y,x, buffer_pos;
+		int y,x, buffer_pos, ymin, ymax;
 
-		buffer_pos = 0;
-		for (y=0; y < vid.height;  y++)
+		ymin = vmin / vid.width;
+		ymax = vmax / vid.width;
+
+		buffer_pos = ymin * vid.width;
+		pixels += ymin * pitch;
+		for (y=ymin; y < ymax;  y++)
 		{
 			for (x=0; x < vid.width; x ++)
 			{
@@ -1739,30 +1996,140 @@ RE_CopyFrame (Uint32 * pixels, int pitch)
 	}
 }
 
-/*
-** RE_EndFrame
-**
-** This does an implementation specific copy from the backbuffer to the
-** front buffer.  In the Win32 case it uses BitBlt or BltFast depending
-** on whether we're using DIB sections/GDI or DDRAW.
-*/
+static int
+RE_BufferDifferenceStart(int vmin, int vmax)
+{
+	int *front_buffer, *back_buffer, *back_max;
+
+	back_buffer = (int*)(swap_frames[0] + vmin);
+	front_buffer = (int*)(swap_frames[1] + vmin);
+	back_max = (int*)(swap_frames[0] + vmax);
+
+	while (back_buffer < back_max && *back_buffer == *front_buffer) {
+		back_buffer ++;
+		front_buffer ++;
+	}
+	return (pixel_t*)back_buffer - swap_frames[0];
+}
+
+static int
+RE_BufferDifferenceEnd(int vmin, int vmax)
+{
+	int *front_buffer, *back_buffer, *back_min;
+
+	back_buffer = (int*)(swap_frames[0] + vmax);
+	front_buffer = (int*)(swap_frames[1] + vmax);
+	back_min = (int*)(swap_frames[0] + vmin);
+
+	do {
+		back_buffer --;
+		front_buffer --;
+	} while (back_buffer > back_min && *back_buffer == *front_buffer);
+        // +1 for fully cover changes
+	return (pixel_t*)back_buffer - swap_frames[0] + sizeof(int);
+}
 
 static void
-RE_EndFrame (void)
+RE_CleanFrame(void)
 {
 	int pitch;
-	Uint32 * pixels;
+	Uint32 *pixels;
+
+	memset(swap_buffers, 0, vid.height * vid.width * sizeof(pixel_t) * 2);
 
 	if (SDL_LockTexture(texture, NULL, (void**)&pixels, &pitch))
 	{
 		Com_Printf("Can't lock texture: %s\n", SDL_GetError());
 		return;
 	}
-	RE_CopyFrame (pixels, pitch / sizeof(Uint32));
+	// only cleanup texture without flush texture to screen
+	memset(pixels, 0, pitch * vid.height);
+	SDL_UnlockTexture(texture);
+
+	// All changes flushed
+	VID_NoDamageBuffer();
+}
+
+static void
+RE_FlushFrame(int vmin, int vmax)
+{
+	int pitch;
+	Uint32 *pixels;
+
+	if (SDL_LockTexture(texture, NULL, (void**)&pixels, &pitch))
+	{
+		Com_Printf("Can't lock texture: %s\n", SDL_GetError());
+		return;
+	}
+	RE_CopyFrame (pixels, pitch / sizeof(Uint32), vmin, vmax);
 	SDL_UnlockTexture(texture);
 
 	SDL_RenderCopy(renderer, texture, NULL, NULL);
 	SDL_RenderPresent(renderer);
+
+	// replace use next buffer
+	swap_current ++;
+	vid_buffer = swap_frames[swap_current&1];
+
+	// All changes flushed
+	VID_NoDamageBuffer();
+}
+
+/*
+** RE_EndFrame
+**
+** This does an implementation specific copy from the backbuffer to the
+** front buffer.
+*/
+static void
+RE_EndFrame (void)
+{
+	int vmin, vmax;
+
+	// fix possible issue with min/max
+	if (vid_minu < 0)
+	{
+		vid_minu = 0;
+	}
+	if (vid_minv < 0)
+	{
+		vid_minv = 0;
+	}
+	if (vid_maxu > vid.width)
+	{
+		vid_maxu = vid.width;
+	}
+	if (vid_maxv > vid.height)
+	{
+		vid_maxv = vid.height;
+	}
+
+	vmin = vid_minu + vid_minv  * vid.width;
+	vmax = vid_maxu + vid_maxv  * vid.width;
+
+	// fix to correct limit
+	if (vmax > (vid.height * vid.width))
+	{
+		vmax = vid.height * vid.width;
+	}
+
+	// search real begin/end of difference
+	vmin = RE_BufferDifferenceStart(vmin, vmax);
+
+	// no differences found
+	if (vmin >= vmax)
+	{
+		return;
+	}
+
+	// search difference end
+	vmax = RE_BufferDifferenceEnd(vmin, vmax);
+	if (vmax > (vid.height * vid.width))
+	{
+		vmax = vid.height * vid.width;
+	}
+
+	RE_FlushFrame(vmin, vmax);
 }
 
 /*
@@ -1807,7 +2174,19 @@ SWimp_SetMode(int *pwidth, int *pheight, int mode, int fullscreen )
 static void
 SWimp_CreateRender(void)
 {
-	vid_buffer = malloc(vid.height * vid.width * sizeof(pixel_t));
+	swap_current = 0;
+	swap_buffers = malloc(vid.height * vid.width * sizeof(pixel_t) * 2);
+	if (!swap_buffers)
+	{
+		ri.Sys_Error(ERR_FATAL, "%s: Can't allocate swapbuffer.", __func__);
+		// code never returns after ERR_FATAL
+		return;
+	}
+	swap_frames[0] = swap_buffers;
+	swap_frames[1] = swap_buffers + vid.height * vid.width * sizeof(pixel_t);
+	vid_buffer = swap_frames[swap_current&1];
+	// Need to rewrite whole frame
+	VID_WholeDamageBuffer();
 
 	sintable = malloc((vid.width+CYCLE) * sizeof(int));
 	intsintable = malloc((vid.width+CYCLE) * sizeof(int));
@@ -1819,17 +2198,27 @@ SWimp_CreateRender(void)
 	warp_rowptr = malloc((vid.width+AMP2*2) * sizeof(byte*));
 	warp_column = malloc((vid.width+AMP2*2) * sizeof(int));
 
-	edge_basespans = malloc((vid.width*2) * sizeof(espan_t));
-
 	// count of "out of items"
-	r_outofsurfaces = r_outofedges = r_outofverts = r_outoftriangles = false;
+	r_outofsurfaces = false;
+	r_outofedges = false;
+	r_outofverts = false;
+	r_outoftriangles = false;
+	r_outoflights = false;
+	r_outedgebasespans = false;
 	// pointers to allocated buffers
 	finalverts = NULL;
 	r_edges = NULL;
 	lsurfs = NULL;
 	triangle_spans = NULL;
+	blocklights = NULL;
+	edge_basespans = NULL;
 	// curently allocated items
-	r_cnumsurfs = r_numallocatededges = r_numallocatedverts = r_numallocatedtriangles = 0;
+	r_cnumsurfs = 0;
+	r_numallocatededges = 0;
+	r_numallocatedverts = 0;
+	r_numallocatedtriangles = 0;
+	r_numallocatedlights = 0;
+	r_numallocatededgebasespans = 0;
 
 	R_ReallocateMapBuffers();
 
