@@ -210,6 +210,7 @@ Mod_PointInLeaf (vec3_t p, model_t *model)
 	if (!model || !model->nodes)
 	{
 		ri.Sys_Error(ERR_DROP, "%s: bad model", __func__);
+		return NULL;
 	}
 
 	node = model->nodes;
@@ -494,15 +495,9 @@ Mod_LoadTexinfo (lump_t *l)
 		}
 		len1 = VectorLength (out->vecs[0]);
 		len2 = VectorLength (out->vecs[1]);
-		len1 = (len1 + len2)/2;
-		if (len1 < 0.32)
-			out->mipadjust = 4;
-		else if (len1 < 0.49)
-			out->mipadjust = 3;
-		else if (len1 < 0.99)
-			out->mipadjust = 2;
-		else
-			out->mipadjust = 1;
+		out->mipadjust = sqrt(len1*len1 + len2*len2);
+		if (out->mipadjust < 0.01)
+			out->mipadjust = 0.01;
 
 		out->flags = LittleLong (in->flags);
 
@@ -988,7 +983,7 @@ Mod_LoadBrushModel(model_t *mod, void *buffer, int modfilelen)
 	int hunkSize = 0;
 	hunkSize += calcLumpHunkSize(&header->lumps[LUMP_VERTEXES], sizeof(dvertex_t), sizeof(mvertex_t), 8);
 	hunkSize += calcLumpHunkSize(&header->lumps[LUMP_EDGES], sizeof(dedge_t), sizeof(medge_t), 13);
-	float surfEdgeCount = header->lumps[LUMP_SURFEDGES].filelen/sizeof(int);
+	float surfEdgeCount = (float)header->lumps[LUMP_SURFEDGES].filelen / sizeof(int);
 	if(surfEdgeCount < MAX_MAP_SURFEDGES) // else it errors out later anyway
 		hunkSize += calcLumpHunkSize(&header->lumps[LUMP_SURFEDGES], sizeof(int), sizeof(int), 24);
 
@@ -1226,6 +1221,8 @@ SPRITE MODELS
 /*
 =================
 Mod_LoadSpriteModel
+
+support for .sp2 sprites
 =================
 */
 static void
