@@ -569,6 +569,7 @@ R_PolysetDrawSpans8_66(const entity_t *currententity, spanpackage_t *pspanpackag
 		if (lcount > 0)
 		{
 			int	pos_shift = (pspanpackage->v * vid.width) + pspanpackage->u;
+			qboolean	zdamaged = false;
 
 			lpdest = d_viewbuffer + pos_shift;
 			lpz = d_pzbuffer + pos_shift;
@@ -586,6 +587,7 @@ R_PolysetDrawSpans8_66(const entity_t *currententity, spanpackage_t *pspanpackag
 
 					*lpdest = vid_alphamap[temp*256 + *lpdest];
 					*lpz = lzi >> SHIFT16XYZ;
+					zdamaged = true;
 				}
 				lpdest++;
 				lzi += r_zistepx;
@@ -602,6 +604,15 @@ R_PolysetDrawSpans8_66(const entity_t *currententity, spanpackage_t *pspanpackag
 					ltfrac &= 0xFFFF;
 				}
 			} while (--lcount);
+
+			if (zdamaged)
+			{
+				// damaged only current line
+				VID_DamageZBuffer(pspanpackage->u, pspanpackage->v);
+				VID_DamageZBuffer(
+					pspanpackage->u + d_aspancount - pspanpackage->count,
+					pspanpackage->v);
+			}
 		}
 
 		pspanpackage++;
@@ -634,6 +645,7 @@ R_PolysetDrawSpansConstant8_66(const entity_t *currententity, spanpackage_t *psp
 		if (lcount > 0)
 		{
 			int	pos_shift = (pspanpackage->v * vid.width) + pspanpackage->u;
+			qboolean	zdamaged = false;
 
 			lpdest = d_viewbuffer + pos_shift;
 			lpz = d_pzbuffer + pos_shift;
@@ -644,11 +656,21 @@ R_PolysetDrawSpansConstant8_66(const entity_t *currententity, spanpackage_t *psp
 				if ((lzi >> SHIFT16XYZ) >= *lpz)
 				{
 					*lpdest = vid_alphamap[r_aliasblendcolor*256 + *lpdest];
+					zdamaged = true;
 				}
 				lpdest++;
 				lzi += r_zistepx;
 				lpz++;
 			} while (--lcount);
+
+			if (zdamaged)
+			{
+				// damaged only current line
+				VID_DamageZBuffer(pspanpackage->u, pspanpackage->v);
+				VID_DamageZBuffer(
+					pspanpackage->u + d_aspancount - pspanpackage->count,
+					pspanpackage->v);
+			}
 		}
 
 		pspanpackage++;
@@ -682,6 +704,7 @@ R_PolysetDrawSpans8_Opaque (const entity_t *currententity, spanpackage_t *pspanp
 			zvalue_t	lzi;
 			zvalue_t	*lpz;
 			int		pos_shift = (pspanpackage->v * vid.width) + pspanpackage->u;
+			qboolean	zdamaged = false;
 
 			lpdest = d_viewbuffer + pos_shift;
 			lpz = d_pzbuffer + pos_shift;
@@ -702,6 +725,7 @@ R_PolysetDrawSpans8_Opaque (const entity_t *currententity, spanpackage_t *pspanp
 						*lpdest = ((byte *)vid_colormap)[*lptex + (llight & 0xFF00)];
 
 					*lpz = lzi >> SHIFT16XYZ;
+					zdamaged = true;
 				}
 				lpdest++;
 				lzi += r_zistepx;
@@ -718,6 +742,15 @@ R_PolysetDrawSpans8_Opaque (const entity_t *currententity, spanpackage_t *pspanp
 					ltfrac &= 0xFFFF;
 				}
 			} while (--lcount);
+
+			if (zdamaged)
+			{
+				// damaged only current line
+				VID_DamageZBuffer(pspanpackage->u, pspanpackage->v);
+				VID_DamageZBuffer(
+					pspanpackage->u + d_aspancount - pspanpackage->count,
+					pspanpackage->v);
+			}
 		}
 
 		pspanpackage++;
