@@ -55,6 +55,9 @@ const char *VID_MenuKey(int);
 // Stuff provided by platform backend.
 extern int glimp_refreshRate;
 
+const char **GLimp_GetDisplayIndices(void);
+int GLimp_GetWindowDisplayIndex(void);
+int GLimp_GetNumVideoDisplays(void);
 qboolean GLimp_Init(void);
 void GLimp_Shutdown(void);
 qboolean GLimp_InitGraphics(int fullscreen, int *pwidth, int *pheight);
