@@ -379,9 +379,11 @@ Cmd_Exec_f(void)
 	Com_Printf("execing %s\n", Cmd_Argv(1));
 
 	/* the file doesn't have a trailing 0, so we need to copy it off */
-	f2 = Z_Malloc(len + 1);
+	/* we also add a newline */
+	f2 = Z_Malloc(len + 2);
 	memcpy(f2, f, len);
-	f2[len] = 0;
+	f2[len] = '\n'; // make sure last line has a newline
+	f2[len+1] = '\0';
 
 	Cbuf_InsertText(f2);
 
