@@ -953,14 +953,19 @@ walkmonster_start_go(edict_t *self)
 		self->yaw_speed = 20;
 	}
 
-	self->viewheight = 25;
-
-	monster_start_go(self);
+	if (!self->viewheight)
+	{
+		self->viewheight = 25;
+	}
 
 	if (self->spawnflags & 2)
 	{
 		monster_triggered_start(self);
 	}
+	else
+	{
+		monster_start_go(self);
+	}
 }
 
 void
@@ -993,14 +998,19 @@ flymonster_start_go(edict_t *self)
 		self->yaw_speed = 10;
 	}
 
-	self->viewheight = 25;
-
-	monster_start_go(self);
+	if (!self->viewheight)
+	{
+		self->viewheight = 25;
+	}
 
 	if (self->spawnflags & 2)
 	{
 		monster_triggered_start(self);
 	}
+	else
+	{
+		monster_start_go(self);
+	}
 }
 
 void
@@ -1029,14 +1039,19 @@ swimmonster_start_go(edict_t *self)
 		self->yaw_speed = 10;
 	}
 
-	self->viewheight = 10;
-
-	monster_start_go(self);
+	if (!self->viewheight)
+	{
+		self->viewheight = 10;
+	}
 
 	if (self->spawnflags & 2)
 	{
 		monster_triggered_start(self);
 	}
+	else
+	{
+		monster_start_go(self);
+	}
 }
 
 void
