@@ -71,7 +71,7 @@
  * load older savegames. This should be bumped if the files
  * in tables/ are changed, otherwise strange things may happen.
  */
-#define SAVEGAMEVER "YQ2-2"
+#define SAVEGAMEVER "YQ2-3"
 
 #ifndef BUILD_DATE
 #define BUILD_DATE __DATE__
@@ -733,7 +733,7 @@ WriteClient(FILE *f, gclient_t *client)
  * Read the client struct from a file
  */
 void
-ReadClient(FILE *f, gclient_t *client)
+ReadClient(FILE *f, gclient_t *client, short save_ver)
 {
 	field_t *field;
 
@@ -741,7 +741,14 @@ ReadClient(FILE *f, gclient_t *client)
 
 	for (field = clientfields; field->name; field++)
 	{
-		ReadField(f, field, (byte *)client);
+		if (field->save_ver <= save_ver)
+		{
+			ReadField(f, field, (byte *)client);
+		}
+	}
+	if (save_ver < 3)
+	{
+		InitClientResp(client);
 	}
 }
 
@@ -749,10 +756,10 @@ ReadClient(FILE *f, gclient_t *client)
 
 /*
  * Writes the game struct into
- * a file. This is called when
- * ever the games goes to e new
- * level or the user saves the
- * game. Saved informations are:
+ * a file. This is called whenever
+ * the game goes to a new level or
+ * the user saves the game. The saved
+ * information consists of:
  * - cross level data
  * - client states
  * - help computer info
@@ -822,6 +829,8 @@ ReadGame(const char *filename)
 	char str_os[32];
 	char str_arch[32];
 
+	short save_ver = 0;
+
 	gi.FreeTags(TAG_GAME);
 
 	f = Q_fopen(filename, "rb");
@@ -839,6 +848,28 @@ ReadGame(const char *filename)
 
 	if (!strcmp(str_ver, SAVEGAMEVER))
 	{
+		save_ver = 3;
+
+		if (strcmp(str_game, GAMEVERSION))
+		{
+			fclose(f);
+			gi.error("Savegame from another game.so.\n");
+		}
+		else if (strcmp(str_os, YQ2OSTYPE))
+		{
+			fclose(f);
+			gi.error("Savegame from another os.\n");
+		}
+		else if (strcmp(str_arch, YQ2ARCH))
+		{
+			fclose(f);
+			gi.error("Savegame from another architecture.\n");
+		}
+	}
+	else if (!strcmp(str_ver, "YQ2-2"))
+	{
+		save_ver = 2;
+
 		if (strcmp(str_game, GAMEVERSION))
 		{
 			fclose(f);
@@ -857,6 +888,8 @@ ReadGame(const char *filename)
 	}
 	else if (!strcmp(str_ver, "YQ2-1"))
 	{
+		save_ver = 1;
+
 		if (strcmp(str_game, GAMEVERSION))
 		{
 			fclose(f);
@@ -901,7 +934,7 @@ ReadGame(const char *filename)
 
 	for (i = 0; i < game.maxclients; i++)
 	{
-		ReadClient(f, &game.clients[i]);
+		ReadClient(f, &game.clients[i], save_ver);
 	}
 
 	fclose(f);
