@@ -389,6 +389,25 @@ SV_ReadServerFile(void)
 
 	ge->ReadGame("game.ssv");
 
+	/* While loading a savegame the global edict arrays is free()ed
+	   and newly malloc()ed to reset all entity states. When the game
+	   puts the first client into the server it sends it's entity
+	   state to us, so as long as there's only one client (the game
+	   is running in single player mode) everything's okay. But when
+	   there're more clients (the game is running in coop mode) the
+	   entity states if all clients >1 are dangeling. hack around
+	   that by reconnecting them here. */
+	cvar_t *coop = Cvar_Get("coop", "0", CVAR_LATCH);
+
+	if (coop->value)
+	{
+		for (int i = 0; i < maxclients->value; i++)
+		{
+			edict_t *ent = EDICT_NUM(i + 1);
+			svs.clients[i].edict = ent;
+		}
+	}
+
 	Sys_SetWorkDir(workdir);
 }
 
